#version 330 core
#extension GL_ARB_explicit_attrib_location: enable

uniform sampler2D terrainTex;
uniform sampler2D terrainTexLinear;
uniform float alphaTest;

in vec4 rgba;
uniform vec4 rgbaFog;
in float fogAmount;
in vec2 uv;
in vec2 uvstem;
in float glowLevel;

flat in int renderFlags;
in vec3 normal;
in vec4 worldPos;
in vec3 vertexPosition;
in vec3 blockLight;
in vec4 gnormal;

uniform float fogDensityIn;
uniform float fogMinIn;
uniform float horizonFog;
uniform vec3 sunPosition;
uniform float dayLight;


layout(location = 0) out vec4 outColor;
layout(location = 1) out vec4 outGlow;
#if SSAOLEVEL > 0
in vec4 fragPosition;
layout(location = 2) out vec4 outGNormal;
layout(location = 3) out vec4 outGPosition;
#endif

#include vertexflagbits.ash
#include fogandlight.fsh
#include dither.fsh
#include skycolor.fsh
#include colormap.fsh

void main() 
{
	// When looking through tinted glass you can clearly see the edges where we fade to sky color
	// Using this discard seems to completely fix that
#if NORMALVIEW == 0		
	if (rgba.a < 0.005) discard;
#endif
	
	vec4 petalColor = texture(terrainTex, uv);
	vec4 texColor = (petalColor.a > 0.005 ? getFrosted(petalColor) : getColorMapped(terrainTexLinear, texture(terrainTex, uvstem))) * rgba;
	
	texColor = applyRustEffect(texColor, normal, vertexPosition, 1);
	
	#if SHADOWQUALITY > 0
	    float intensity = 0.34 + (1 - shadowIntensity)/8.0; // this was 0.45, which makes shadow acne visible on blocks
	#else
	    float intensity = 0.45;
	#endif
	
	outColor = applyFogAndShadowWithNormal(texColor, fogAmount, normal, 1, intensity); 
	
	float glow = 0;
	
	
	// Fade to sky color
	float aTest = outColor.a + max(0, 1 - rgba.a) * min(1, outColor.a * 10);

#if NORMALVIEW == 0	
	if (aTest < alphaTest)
	{
	    discard;
	}
	else
#endif
	{
#if SHINYEFFECT > 0
	glow = pow(max(0, dot(normal, lightPosition)), 6) / 8 * shadowIntensity * (1 - fogAmount);
#endif
	}


#if SSAOLEVEL > 0
	outGPosition = vec4(fragPosition.xyz, fogAmount*2 + glowLevel);
	outGNormal = gnormal;
#endif

#if NORMALVIEW > 0
	outColor = vec4((normal.x + 1) / 2, (normal.y + 1) / 2, (normal.z + 1) / 2, 1);	
#endif
	outGlow = vec4(glowLevel + glow, 0, 0, outColor.a);
	
}
